@@ -1,4 +1,4 @@
-#Copyright 2010-2012 Miquel Torres <tobami@gmail.com>
+#Copyright 2010-2013 Miquel Torres <tobami@gmail.com>
 #
 #Licensed under the Apache License, Version 2.0 (the "License");
 #you may not use this file except in compliance with the License.
@@ -21,7 +21,7 @@ import simplejson as json
 from fabric.api import *
 from fabric.contrib.files import append, exists
 from fabric.contrib.console import confirm
-from ssh.config import SSHConfig as _SSHConfig
+from paramiko.config import SSHConfig as _SSHConfig
 
 import littlechef
 from littlechef import solo, lib, chef
@@ -31,6 +31,7 @@ import fabric
 fabric.state.output['running'] = False
 env.loglevel = littlechef.loglevel
 env.verbose = littlechef.verbose
+env.abort_on_prompts = littlechef.noninteractive
 env.chef_environment = littlechef.chef_environment
 env.node_work_path = littlechef.node_work_path
 
@@ -113,7 +114,7 @@ def node(*nodes):
             message += " in the {0} environment".format(env.chef_environment)
         message += "?"
         if not __testing__:
-            if not confirm(message):
+            if not lib.global_confirm(message):
                 abort('Aborted by user')
     else:
         # A list of nodes was given
@@ -130,6 +131,10 @@ def node(*nodes):
             execute(_node_runner)
         chef.remove_local_node_data_bag()
 
+def _configure_fabric_for_platform(platform):
+    """Configures fabric for a specific platform"""
+    if platform == "freebsd":
+        env.shell = "/bin/sh -c"
 
 def _node_runner():
     """This is only used by node so that we can execute in parallel"""
@@ -138,6 +143,9 @@ def _node_runner():
     if '@' in env.host_string:
         env.user = env.host_string.split('@')[0]
     node = lib.get_node(env.host_string)
+
+    _configure_fabric_for_platform(node.get("platform"))
+
     if __testing__:
         print "TEST: would now configure {0}".format(env.host_string)
     else:
@@ -146,7 +154,7 @@ def _node_runner():
 
 
 def deploy_chef(gems="no", ask="yes", version="0.10",
-    distro_type=None, distro=None, stop_client='yes'):
+    distro_type=None, distro=None, platform=None, stop_client='yes'):
     """Install chef-solo on a node"""
     if not env.host_string:
         abort('no node specified\nUsage: fix node:MYNODES deploy_chef')
@@ -155,29 +163,44 @@ def deploy_chef(gems="no", ask="yes", version="0.10",
         abort('Wrong Chef version specified. Valid versions are {0}'.format(
             ", ".join(chef_versions)))
     if distro_type is None and distro is None:
-        distro_type, distro = solo.check_distro()
+        distro_type, distro, platform = solo.check_distro()
     elif distro_type is None or distro is None:
         abort('Must specify both or neither of distro_type and distro')
-    if ask == "yes":
-        message = '\nAre you sure you want to install Chef {0}'.format(version)
-        message += ' on node {0}'.format(env.host_string)
-        if gems == "yes":
-            message += ', using gems for "{0}"?'.format(distro)
-        else:
-            message += ', using "{0}" packages?'.format(distro)
+    if gems == "yes":
+        method = 'using gems for "{0}"'.format(distro)
+    else:
+        method = '{0} using "{1}" packages'.format(version, distro)
+    if ask == "no" or littlechef.noninteractive:
+        print("Deploying Chef {0}...".format(method))
+    else:
+        message = ('\nAre you sure you want to install Chef '
+                   '{0} on node {1}?'.format(method, env.host_string))
         if not confirm(message):
             abort('Aborted by user')
-    else:
-        if gems == "yes":
-            method = 'using gems for "{0}"'.format(distro)
-        else:
-            method = '{0} using "{1}" packages'.format(version, distro)
-        print("Deploying Chef {0}...".format(method))
+
+    _configure_fabric_for_platform(platform)
 
     if not __testing__:
         solo.install(distro_type, distro, gems, version, stop_client)
         solo.configure()
 
+        # Build a basic node file if there isn't one already
+        # with some properties from ohai
+        with settings(hide('stdout'), warn_only=True):
+            output = sudo('ohai -l warn')
+        if output.succeeded:
+            try:
+                ohai = json.loads(output)
+            except json.JSONDecodeError:
+                abort("Could not parse ohai's output"
+                      ":\n  {0}".format(output))
+            node = {"run_list": []}
+            for prop in ["ipaddress", "platform", "platform_family",
+                         "platform_version"]:
+                if ohai.get(prop):
+                    node[prop] = ohai[prop]
+            chef.save_config(node)
+
 
 def recipe(recipe):
     """Apply the given recipe to a node
