@@ -1,4 +1,4 @@
-#Copyright 2010-2012 Miquel Torres <tobami@gmail.com>
+#Copyright 2010-2013 Miquel Torres <tobami@gmail.com>
 #
 #Licensed under the Apache License, Version 2.0 (the "License");
 #you may not use this file except in compliance with the License.
@@ -22,146 +22,166 @@ from fabric.contrib.files import append, exists, upload_template
 from fabric.utils import abort
 
 from littlechef import cookbook_paths
-from littlechef.lib import credentials
 from littlechef import LOGFILE
 
-
 # Path to local patch
 BASEDIR = os.path.abspath(os.path.dirname(__file__).replace('\\', '/'))
 
 
 def install(distro_type, distro, gems, version, stop_client):
     """Calls the appropriate installation function for the given distro"""
-    with credentials():
-        if distro_type == "debian":
-            if gems == "yes":
-                _gem_apt_install()
-            else:
-                _apt_install(distro, version, stop_client)
-        elif distro_type == "rpm":
-            if gems == "yes":
-                _gem_rpm_install()
-            else:
-                _rpm_install()
-        elif distro_type == "gentoo":
-            _emerge_install()
-        elif distro_type == "pacman":
-            _gem_pacman_install()
+    if distro_type == "debian":
+        if gems == "yes":
+            _gem_apt_install()
         else:
-            abort('wrong distro type: {0}'.format(distro_type))
+            _apt_install(distro, version, stop_client)
+    elif distro_type == "rpm":
+        if gems == "yes":
+            _gem_rpm_install()
+        else:
+            _rpm_install()
+    elif distro_type == "gentoo":
+        _emerge_install()
+    elif distro_type == "pacman":
+        _gem_pacman_install()
+    elif distro_type == "freebsd":
+        _gem_ports_install()
+    else:
+        abort('wrong distro type: {0}'.format(distro_type))
 
 
 def configure(current_node=None):
     """Deploy chef-solo specific files"""
     current_node = current_node or {}
-    with credentials():
-        # Ensure that the /tmp/chef-solo/cache directory exist
-        cache_dir = "{0}/cache".format(env.node_work_path)
-        if not exists(cache_dir):
-            with settings(hide('running', 'stdout'), warn_only=True):
-                output = sudo('mkdir -p {0}'.format(cache_dir))
-            if output.failed:
-                error = "Could not create {0} dir. ".format(env.node_work_path)
-                error += "Do you have sudo rights?"
-                abort(error)
-        # Change ownership of /tmp/chef-solo/ so that we can rsync
-        with hide('running', 'stdout'):
-            with settings(warn_only=True):
-                output = sudo(
-                    'chown -R {0} {1}'.format(env.user, env.node_work_path))
-            if output.failed:
-                error = "Could not modify {0} dir. ".format(env.node_work_path)
-                error += "Do you have sudo rights?"
-                abort(error)
-        # Set up chef solo configuration
-        logging_path = os.path.dirname(LOGFILE)
-        if not exists(logging_path):
-            sudo('mkdir -p {0}'.format(logging_path))
-        if not exists('/etc/chef'):
-            sudo('mkdir -p /etc/chef')
-        # Set parameters and upload solo.rb template
-        reversed_cookbook_paths = cookbook_paths[:]
-        reversed_cookbook_paths.reverse()
-        cookbook_paths_list = '[{0}]'.format(', '.join(
-            ['"{0}/{1}"'.format(env.node_work_path, x) \
-                for x in reversed_cookbook_paths]))
-        data = {
-            'node_work_path': env.node_work_path,
-            'cookbook_paths_list': cookbook_paths_list,
-            'environment': current_node.get('chef_environment', '_default'),
-            'verbose': "true" if env.verbose else "false"
-        }
-        with settings(hide('everything')):
-            try:
-                upload_template(os.path.join(BASEDIR, 'solo.rb'), '/etc/chef/',
-                    context=data, use_sudo=True, backup=False, mode=0400)
-            except SystemExit:
-                error = ("Failed to upload '/etc/chef/solo.rb'\n"
-                "This can happen when the deployment user does not have a "
-                "home directory, which is needed as a temporary location")
-                abort(error)
-        with hide('stdout'):
-            sudo('chown root:root {0}'.format('/etc/chef/solo.rb'))
+    # Ensure that the /tmp/chef-solo/cache directory exist
+    cache_dir = "{0}/cache".format(env.node_work_path)
+    if not exists(cache_dir):
+        with settings(hide('running', 'stdout'), warn_only=True):
+            output = sudo('mkdir -p {0}'.format(cache_dir))
+        if output.failed:
+            error = "Could not create {0} dir. ".format(env.node_work_path)
+            error += "Do you have sudo rights?"
+            abort(error)
+    # Change ownership of /tmp/chef-solo/ so that we can rsync
+    with hide('running', 'stdout'):
+        with settings(warn_only=True):
+            output = sudo(
+                'chown -R {0} {1}'.format(env.user, env.node_work_path))
+        if output.failed:
+            error = "Could not modify {0} dir. ".format(env.node_work_path)
+            error += "Do you have sudo rights?"
+            abort(error)
+    # Set up chef solo configuration
+    logging_path = os.path.dirname(LOGFILE)
+    if not exists(logging_path):
+        sudo('mkdir -p {0}'.format(logging_path))
+    if not exists('/etc/chef'):
+        sudo('mkdir -p /etc/chef')
+    # Set parameters and upload solo.rb template
+    reversed_cookbook_paths = cookbook_paths[:]
+    reversed_cookbook_paths.reverse()
+    cookbook_paths_list = '[{0}]'.format(', '.join(
+        ['"{0}/{1}"'.format(env.node_work_path, x) \
+            for x in reversed_cookbook_paths]))
+    data = {
+        'node_work_path': env.node_work_path,
+        'cookbook_paths_list': cookbook_paths_list,
+        'environment': current_node.get('chef_environment', '_default'),
+        'verbose': "true" if env.verbose else "false"
+    }
+    with settings(hide('everything')):
+        try:
+            upload_template(os.path.join(BASEDIR, 'solo.rb'), '/etc/chef/',
+                context=data, use_sudo=True, backup=False, mode=0400)
+        except SystemExit:
+            error = ("Failed to upload '/etc/chef/solo.rb'\n"
+            "This can happen when the deployment user does not have a "
+            "home directory, which is needed as a temporary location")
+            abort(error)
+    with hide('stdout'):
+        sudo('chown root:$(id -g -n root) {0}'.format('/etc/chef/solo.rb'))
 
 
 def check_distro():
     """Check that the given distro is supported and return the distro type"""
-    debian_distros = ['wheezy', 'squeeze', 'lenny']
-    ubuntu_distros = ['natty', 'maverick', 'lucid', 'karmic']
-    rpm_distros = ['centos', 'rhel', 'sl']
+    def print_supported_distros(platform):
+        supported_distros = (
+            "Currently supported distros are:"
+            " Debian, Ubuntu, RHEL (CentOS, RHEL, SL),"
+            " Gentoo, Arch Linux or FreeBSD")
+        print supported_distros
+        abort("Unsupported distro '{0}'".format(platform))
 
-    with credentials(
-        hide('warnings', 'running', 'stdout', 'stderr'), warn_only=True):
-        output = sudo('cat /etc/issue')
-        if 'Debian GNU/Linux 5.0' in output:
-            distro = "lenny"
-            distro_type = "debian"
-        elif 'Debian GNU/Linux 6.0' in output:
-            distro = "squeeze"
-            distro_type = "debian"
-        elif 'Debian GNU/Linux wheezy' in output:
-            distro = "wheezy"
-            distro_type = "debian"
-        elif 'Ubuntu' in output:
-            distro = sudo('lsb_release -cs')
-            distro_type = "debian"
-        elif 'CentOS' in output:
-            distro = "CentOS"
-            distro_type = "rpm"
-        elif 'Red Hat Enterprise Linux' in output:
-            distro = "Red Hat"
-            distro_type = "rpm"
-        elif 'Scientific Linux SL' in output:
-            distro = "Scientific Linux"
-            distro_type = "rpm"
-        elif 'This is \\n.\\O (\\s \\m \\r) \\t' in output:
-            distro = "Gentoo"
-            distro_type = "gentoo"
-        elif 'Arch Linux \\r  (\\n) (\\l)' in output:
-            distro = "Arch Linux"
-            distro_type = "pacman"
+    with settings(hide('warnings', 'running', 'stdout', 'stderr'),
+                  warn_only=True):
+        # use /bin/sh to determine our OS. FreeBSD doesn't have /bin/bash
+        original_shell = env.shell
+        env.shell = "/bin/sh -c"
+        os_implementation = run('uname -o')
+        if 'Linux' in os_implementation:
+            env.shell = original_shell
+            output = sudo('cat /etc/issue')
+            if 'Debian GNU/Linux 5.0' in output:
+                distro = "lenny"
+                distro_type = "debian"
+                platform = "debian"
+            elif 'Debian GNU/Linux 6.0' in output:
+                distro = "squeeze"
+                distro_type = "debian"
+                platform = "debian"
+            elif 'Debian GNU/Linux 7.0' in output:
+                distro = "wheezy"
+                distro_type = "debian"
+                platform = "debian"
+            elif 'Ubuntu' in output:
+                distro = sudo('lsb_release -cs')
+                distro_type = "debian"
+                platform = "ubuntu"
+            elif 'CentOS' in output:
+                distro = "CentOS"
+                distro_type = "rpm"
+                platform = "centos"
+            elif 'Red Hat Enterprise Linux' in output:
+                distro = "Red Hat"
+                distro_type = "rpm"
+                platform = "redhat"
+            elif 'Scientific Linux' in output:
+                distro = "Scientific Linux"
+                distro_type = "rpm"
+                platform = "scientific"
+            elif 'This is \\n.\\O (\\s \\m \\r) \\t' in output:
+                distro = "Gentoo"
+                distro_type = "gentoo"
+                platform = "gentoo"
+            elif 'Arch Linux \\r  (\\n) (\\l)' in output:
+                distro = "Arch Linux"
+                distro_type = "pacman"
+                platform = "arch"
+            else:
+                print_supported_distros(output)
+        elif 'FreeBSD' in os_implementation:
+            env.shell = "/bin/sh -c"
+            distro = "FreeBSD"
+            distro_type = "freebsd"
+            platform = "freebsd"
         else:
-            print "Currently supported distros are:"
-            print "  Debian: " + ", ".join(debian_distros)
-            print "  Ubuntu: " + ", ".join(ubuntu_distros)
-            print "  RHEL: " + ", ".join(rpm_distros)
-            print "  Gentoo"
-            print "  Arch Linux"
-            abort("Unsupported distro '{0}'".format(output))
-    return distro_type, distro
+            print_supported_distros(os_implementation)
+
+    return distro_type, distro, platform
 
 
 def _gem_install():
     """Install Chef from gems"""
     # Install RubyGems from Source
     rubygems_version = "1.8.10"
+    ruby_version = "'~> 10.0'"
     run('wget http://production.cf.rubygems.org/rubygems/rubygems-{0}.tgz'
         .format(rubygems_version))
     run('tar zxf rubygems-{0}.tgz'.format(rubygems_version))
     with cd('rubygems-{0}'.format(rubygems_version)):
         sudo('ruby setup.rb --no-format-executable'.format(rubygems_version))
     sudo('rm -rf rubygems-{0} rubygems-{0}.tgz'.format(rubygems_version))
-    sudo('gem install --no-rdoc --no-ri chef')
+    sudo('gem install --no-rdoc --no-ri chef -v {0}'.format(ruby_version))
 
 
 def _gem_apt_install():
@@ -192,6 +212,16 @@ def _gem_pacman_install():
         sudo('pacman -S --noconfirm ruby base-devel wget rsync')
     sudo('gem install --no-rdoc --no-ri chef')
 
+def _gem_ports_install():
+    """Install Chef from gems for FreeBSD"""
+    with hide('stdout', 'running'):
+        sudo('grep -q RUBY_VER /etc/make.conf || echo \'RUBY_VER=1.9\' >> /etc/make.conf')
+        sudo('grep -q RUBY_DEFAULT_VER /etc/make.conf || echo \'RUBY_DEFAULT_VER=1.9\' >> /etc/make.conf')
+    with show('running'):
+        sudo('which -s rsync || pkg_add -r rsync')
+        sudo('which -s perl || pkg_add -r perl')
+        sudo('which -s m4 || pkg_add -r m4')
+        sudo('which -s chef || (cd /usr/ports/sysutils/rubygem-chef && make -DBATCH install)')
 
 def _apt_install(distro, version, stop_client='yes'):
     """Install Chef for debian based distros"""
